function [EST,TAB] = c2BayesSeriesScale(X,j1,j2,Nwt,eta,gamint)
%% function [EST,TAB] = c2BayesSeriesScale(X,j1,j2,Nwt,eta,gamint)
%
% Bayesian joint estimation of the multifractality parameter c2 and of the integral scale J for time series
%
% INPUT:
%   X       time series to be analyzed
%   j1      finest scale to use in estimation [default: 1]
%   j2      coarsest scale to use in estimation [default: log2(length(im))-4]
%   Nwt     number of vanishing moments of wavelet
%   eta     bandwidth parameter for Whittle approximation [default and recommended: 1]
%   gamint  fractional integration by order gamint to ensure minimum
%           regularity of image [default: 0]
%
% OUTPUT:
%   EST.c2 and EST.c20
%       structure with estimates of c2 (and intersept c20) obtained with
%       .MMSE   Bayesian MMSE estimator
%       .MAP    Bayesian MAP estimator
%       .LS     least-squares estimator
%       .wLS    weighted least-squares estimator
%   EST.J
%       structure with estimates of Integral Scale J obtained with
%       .MMSE   Bayesian MMSE estimator
%       .MAP    Bayesian MAP estimator
%   TAB
%       structure with samples generated by MCMC (.c2 and .c20 .J)
%       and corresponding log-posterior values (.critc2 and .critc20 .J)
%       can be used to monitor convergence:
%           iteration number is hard-wired on line 71 below
%
%--- Sebastien Combrexelle, Toulouse, 2013-2015
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% If you use this code, please quote:
%
% @INPROCEEDINGS{CombrexelleICASSP15,
%   author = {Sebastien Combrexelle and Herwig Wendt and Patrice Abry and Nicolas Dobigeon and Stephen McLaughlin and Jean-Yves Tourneret},
%   title = {A Bayesian approach for the joint estimation of the multifractality parameter and integral scale based on the Whittle approximation},
%   booktitle = {IEEE Int. Conf. Acoust., Speech, and Signal Proc. (ICASSP)},
%   address = {Brisbane, Australia},
%   month = {April},
%   year = {2015}
% }
%
% and
%
% @INPROCEEDINGS{WendtICASSP2013,
%   author = {Herwig Wendt and Nicolas Dobigeon and Jean-Yves Tourneret and Patrice Abry},
%   title = {Bayesian estimation for the multifractality parameter},
%   booktitle = {IEEE Int. Conf. Acoust., Speech, and Signal Proc. (ICASSP)},
%   address = {Vancouver, Canada},
%   month = {September},
%   year = {2013}
% }
%
% Please refer to these references for details.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if nargin < 6; gamint = 0;end;
if nargin < 5; eta = 1;end;
if nargin < 4; Nwt = 2;end
if nargin < 3; j2 = max(2,log2(length(X))-4);end
if nargin < 2; j1 = 1; end
%keyboard
% Periodogram
[IYm,Ni,Nj,ind,VV] = PrepPerioSerie(X,Nwt,j1,j2,eta,gamint);
% PSD function storage
[psd,LLc] = psd_bayes_serieScale(-0.04,0.3,j1,j2,Nj,ind);
% Sampling
N=length(X); Nmc=min(8000,max(2000,1/2*(log2(N)-8)*1000)); Nbi=round(Nmc/2);
%[EST,TAB] = SampleGibbsRWScale(IYm,LLc,j1,j2,Ni,Nj,ind, Nmc, Nbi);
[EST,TAB] = SampleGibbsRWScaleICASSP(IYm,LLc,j1,j2,Ni,Nj,ind, Nmc, Nbi);
% least squares estimates
[EST.c2.LS,EST.c20.LS] = regr_cp(VV,j1,j2,Nj(:,2)',0);
[EST.c2.wLS,EST.c20.wLS] = regr_cp(VV,j1,j2,Nj(:,2)',1);
end


%% inline functions
function [EST,TAB] = SampleGibbsRWScaleICASSP(IYm,FUN,j1,j2,Ni,Nj,ind,Nmc,Nbi)
if nargin<9; Nbi = 1000; end
if nargin<8; Nmc = 2000; end
% Parameters
walkvarcom = 0.04; NbLeaders = sum(prod(Nj,2));
walkvar_c2 = walkvarcom^.5/sqrt(NbLeaders);walkvar_c20 = 3.7*walkvar_c2;walkvar_ta = walkvar_c20/1.2;
Mj1 = [j1*log(2) 1] ;Mj2 = [j2*log(2) 1];
% Initialization
nwind = 50; Vacc_c2 = randn(1,nwind)<0.0;Vacc_c20 = randn(1,nwind)<0.0;Vacc_ta = randn(1,nwind);
c2  = -0.04;c20 = 0.3; ta = 0.2;
C_est = [c2;c20;ta]; tabC = zeros(3,Nmc);
rndn = randn(Nmc,3);rndn = [rndn,rand(Nmc,3)];
PSD = (C_est(1:2)'*FUN);IND = PSD > 0; PSD = abs(PSD);
U = 0.5*(sum(IND.*Ni.*log(PSD))+sum(IND.*IYm./PSD));
Crit=zeros(1,Nmc+1);Crit0=zeros(1,Nmc+1);CritJ=zeros(1,Nmc+1);
alpha = sqrt(2);
%% Iterations
for t=1:Nmc
    % Tuning and control of step sizes
    mVacc_c2(t) = mean(Vacc_c2); mVacc_c20(t) = mean(Vacc_c20);mVacc_ta(t) = mean(Vacc_ta);
    if mVacc_c2(t)<0.4;
        Vacc_c2 = randn(1,nwind)<0.0;walkvar_c2 = walkvar_c2/alpha;
    else
        if mVacc_c2(t)>0.6
            Vacc_c2 = randn(1,nwind)<0.0;walkvar_c2 = walkvar_c2*alpha;
        end
    end
    if mVacc_c20(t)<0.4;
        Vacc_c20 = randn(1,nwind)<0.0;walkvar_c20 = walkvar_c20/alpha;
    else
        if mVacc_c20(t)>0.6
            Vacc_c20 = randn(1,nwind)<0.0;walkvar_c20 = walkvar_c20*alpha;
        end
    end
    if mVacc_ta(t)<0.4;
        Vacc_ta = randn(1,nwind)<0.0;walkvar_ta = walkvar_ta/alpha;
    else
        if mVacc_ta(t)>0.6
            Vacc_ta = randn(1,nwind)<0.0;walkvar_ta = walkvar_ta*alpha;
        end
    end
    % Update c2
    Ccand  = C_est + [1;0;0]*walkvar_c2*rndn(t,1);
    if (Mj1*Ccand(1:2)<0) || (Mj2*Ccand(1:2)<0)
        Vacc_c2 = [Vacc_c2(2:end) 0];
        Crit(t+1)=Crit(t);
    else
        FFTcand = Functions_FFT_serieScale(j1,j2,Nj,ind,Ccand(1),Ccand(2),Ccand(3));
        Lc2 = [];Lc20 = [];
        for j=j1:j2;Lc2 = [Lc2 FFTcand.c2{j}(:)'];Lc20=[Lc20 FFTcand.c20{j}(:)'];end
        FUNcand = [Lc2;Lc20];
        PSDcand = (Ccand(1:2)'*FUNcand); PSDcand = abs(PSDcand);INDcand = PSDcand > 0;
        Ucand = 0.5*(sum(INDcand.*Ni.*log(PSDcand))+sum(INDcand.*IYm./PSDcand));
        if rndn(t,4)<min(1,exp((U) - (Ucand)))   % Test
            C_est(1) = Ccand(1);U = Ucand;Vacc_c2= [Vacc_c2(2:end) 1];
            FUN = FUNcand; % Stock new PSD contributions c2/c20 for the accepted c2
        else
            Vacc_c2 = [Vacc_c2(2:end) 0];
        end
        Crit(t+1)=U;
    end
    % Update c20
    Ccand  = C_est + [0;1;0]*walkvar_c20*rndn(t,2);
    if (Mj1*Ccand(1:2)<0) || (Mj2*Ccand(1:2)<0)
        Vacc_c20 = [Vacc_c20(2:end) 0];
        Crit0(t+1)=Crit0(t);
    else
        PSDcand = (Ccand(1:2)'*FUN); INDcand = PSDcand > 0; PSDcand = abs(PSDcand);
        Ucand = 0.5*(sum(INDcand.*Ni.*log(PSDcand))+sum(INDcand.*IYm./PSDcand));
        if rndn(t,5)<min(1,exp((U) - (Ucand)))   % Test
            C_est(2) = Ccand(2);U = Ucand;Vacc_c20= [Vacc_c20(2:end) 1];
        else
            Vacc_c20 = [Vacc_c20(2:end) 0];
        end
        Crit0(t+1)=U;
    end
    % Update eta (possibly add a uniform prior on a set)
    Ccand  = C_est + [0;0;1]*walkvar_ta*rndn(t,3);
    FFTcand = Functions_FFT_serieScale(j1,j2,Nj,ind,Ccand(1),Ccand(2),Ccand(3));
    Lc2 = [];Lc20 = [];
    for j=j1:j2;Lc2 = [Lc2 FFTcand.c2{j}(:)'];Lc20=[Lc20 FFTcand.c20{j}(:)'];end
    FUNcand = [Lc2;Lc20];
    PSDcand = (Ccand(1:2)'*FUNcand); INDcand = PSDcand > 0; PSDcand = abs(PSDcand);
    Ucand = 0.5*(sum(INDcand.*Ni.*log(PSDcand))+sum(INDcand.*IYm./PSDcand));
    if rndn(t,6)<min(1,exp((U) - (Ucand)))   % Test
        C_est(3) = Ccand(3);
        U = Ucand;Vacc_ta= [Vacc_ta(2:end) 1];
        FUN = FUNcand; % Stock new PSD contributions c2/c20 for the accepted J
    else
        Vacc_ta = [Vacc_ta(2:end) 0];
    end
    CritJ(t+1)=U;
    tabC(:,t) = C_est;
    tabW(:,t) = [walkvar_c2;walkvar_c20;walkvar_ta];
end
% - OUTPUTS
EST.c2.MMSE = mean(tabC(1,Nbi:end));EST.c20.MMSE = mean(tabC(2,Nbi:end));
% EST.ta.MMSE = mean(tabC(3,Nbi:end));EST.J.MMSE = -EST.ta.MMSE/(log(2)*EST.c2.MMSE);
EST.J.MMSE = -mean(tabC(3,Nbi:end))/(log(2)*EST.c2.MMSE);
TAB.c2 = tabC(1,:);TAB.c20=tabC(2,:);
% TAB.ta=tabC(3,:);TAB.J = -TAB.ta./TAB.c2/log(2);
TAB.J = -tabC(3,:)./TAB.c2/log(2);
% TAB.W = tabW;
TAB.critc2=-Crit(2:end); [tmp,id]=max(-Crit(2:end)); EST.c2.MAP=tabC(1,id);
TAB.critc20=-Crit0(2:end); [tmp,id]=max(-Crit0(2:end)); EST.c20.MAP=tabC(2,id);
TAB.critJ=-CritJ(2:end); [tmp,id]=max(-CritJ(2:end)); EST.J.MAP=TAB.J(id);
end

function [IYm,Ni,Nj,ind,VV] = PrepPerioSerie(X,Nwt,j1,j2,eta,gamint)
[coef, leaders, nj] = DxLx1d_bayes_dwt(X,Nwt,gamint);
for j=j1:j2; lx{j} = log(leaders(j).value); VV(j)=var(lx{j}); Nj(j,:) = size(lx{j}); Sdx(j)=log2(coef(j).supcoef); end
[hminint] = regr_cp(Sdx,j1,j2,Nj(:,2)',0);
if hminint<=0
    warning(['insufficient minimum regularity (hmin=',num2str(round(10*hminint)/10),'<0): increase gamint by ',num2str(ceil(-2*hminint)/2)])
end
[w,ind] = fft_freq(j1,j2,Nj,eta);
IYv = []; Ni = [];
for j = j1:j2
    IY = abs(fftshift(fftn(lx{j}-mean(lx{j}(:))))).^2; IY = IY/prod(Nj(j,:));
    tp = IY(ind{j}); IYv = [IYv tp(:)'];
end
Ni = ones(size(IYv)); IYm(1,:)=IYv;
end


function [psd,LLc] = psd_bayes_serieScale(c2,c20,j1,j2,Nj,ind)
% COMPUTE PSD VIA FFT OF THE COVARIANCE
FUN = Functions_FFT_serieScale(j1,j2,Nj,ind,c2,c20,0.2);
for j = j1:j2; psd{j} = abs(c2*FUN.c2{j}+c20*FUN.c20{j});end
% Concatenation in a unique vector
Lc2 = [];Lc20 = [];
for j=j1:j2;Lc2 = [Lc2 FUN.c2{j}(:)'];Lc20=[Lc20 FUN.c20{j}(:)'];end
LLc = [Lc2;Lc20];
end

function FUN = Functions_FFT_serieScale(j1,j2,Nj,ind,c2,c20,ta)
rc = 3; k = 4; Nj = prod(Nj,2);
if nargin <7 ; r0(j1:j2) = (Nj(j1:j2)/k);else; r0(j1:j2) = (2.^(-((j1:j2)+ta/c2/log(2))));end
for j=j1:j2
    % Build full symmetric covariance
    x  = (-(Nj(j)):(Nj(j)-1)); r  = abs(x); rj = r0(j);
    covL = c2*log((r)/rj); covL = max(covL,0); % Asymptotic covariance model
    aj = (c2*log((1+rc)/rj)-c20-c2*j*log(2))/(log(1+rc));covS = aj*log(r)+c20+c2*j*log(2);indS = (r<=rc); indL = (r>rc);% Short-covariance
    %cov.full{j} = covL; cov.full{j}(indS) = covS(indS); % Full covariance
    % Separate contributions of c2 and c20 and store correspondant functions
    rcovL = min(0,log(r/rj)); rcovL(indS) = 0;
    rcovSc2 = (log(rc/(rj*2^j))/log(1+rc))*log(1+r)+j*log(2); rcovSc2(indL)=0;
    rcovSc20= 1-log(1+r)/log(1+rc); rcovSc20(indL) = 0;
    cov.c2{j} = rcovL +rcovSc2;cov.c20{j} = rcovSc20;
    cov.full{j} = c2*cov.c2{j} + c20*cov.c20{j};
    % Separate FFT on the two contributions
    tpc2 = fftshift(fft(cov.c2{j}));tpc20 = fftshift(fft(cov.c20{j})); %FFT
    tpc2 = tpc2(1:2:end);tpc20 = tpc20(1:2:end); % Decimation + vectorization
    IND = ind{j}; FUN.c2{j} = tpc2(IND); FUN.c20{j} = tpc20(IND);  % Bandwidth selection
end
end

function [w,ind] = fft_freq(j1,j2,Nj,eta)
% Compute frequencies used in fft
for j=j1:j2
    fs = 2*pi;
    df1 = fs/Nj(j,1); wk1 = (0:df1:(fs-df1)) - (fs-mod(Nj(j,1),2)*df1)/2;
    df2 = fs/Nj(j,2); wk2 = (0:df2:(fs-df2)) - (fs-mod(Nj(j,2),2)*df2)/2;
    [wx,wy] = meshgrid(wk1,wk2);wr = sqrt(wx.^2+wy.^2);
    ind{j} = (wr<=sqrt(eta)*pi)&(wr>0.01);
    w.x{j} = wx(ind{j}); % First composant
    w.y{j} = wy(ind{j}); % Second composant
    w.r{j} = wr(ind{j}); % Radius in 2D / absolute value in 1D
end
end

function [coef, leaders, nj] = DxLx1d_bayes_dwt(data, Nwt, gamint, Jmax)
%% function [coef, leaders, nj] = DWTc2Bayes(data, Nwt, gamint, Jmax)
Norm=1;
if nargin<3; gamint=0; Jmax=Inf; end;
if nargin<4; Jmax=Inf; end;
if nargout==1; doLeaders=0; else; doLeaders=1; end
p=Inf;
%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% DWT
%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%-- Initialize the wavelet filters
n = length(data) ;                   % data length
u = rlistcoefdaub(Nwt) ; nu = 0 ;
[v,nv] = tildeurflippeur(u,nu);  nv = nv + 1;
nls=length(v)/2+1; doff=-1;
[hh1,nh1] = flippeur(u,nu) ; nl=length(hh1);
[gg1,ng1] = flippeur(v,nv) ;
%--- Predict the max # of octaves available given Nwt, and take the min with
nbvoies= fix( log2(length(data)) );
nbvoies = min( Jmax  , nbvoies);  %   safer, casadestime having problems
nbvoies = min( fix(log2(n/(nl+1)))  , nbvoies);  %   safer, casadestime having problems
nbvoies=min(nbvoies,Jmax);
%--- Compute the WT, calculate statistics
njtemp = n;    %  this is the the appro, changes at each scale
approW=data;
approL=data;
for j=1:nbvoies         % Loop Scales
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%% PHASE 1: COEFFICIENTS, LEADERS
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %-- Phase 1a: get the wavelet coefficients at this scale
    njtemp = length(approW) ;
    conv_gg1 = conv(approW,gg1) ;
    conv_hh1 = conv(approW,hh1) ;
    approW = conv_hh1(nls:2:njtemp+nls-1) ; %-- prepare for Phase 1 at next coarser scale
    decime = conv_gg1(nls:2:njtemp+nls-1) ;   % details at scale j. decime always becomes empty near the end,
    nj.W(j) = length(decime);            % number of coefficients
    if ~doLeaders
        coef(j).nj=nj.W(j);
    end
    AbsdqkW = abs(decime)*2^(j/2)/2^(j/Norm);   %%%%%%% passage Norme L1
    % fractional integration
    AbsdqkW = AbsdqkW*2^(gamint*j);
    coef(j).value_sbord=(AbsdqkW)*2^(j/2)/2^(j/Norm);
    coef(j).nbord(1)=nls+doff; coef(j).nbord(2)=length(AbsdqkW)-nls+2;
    coef(j).value=AbsdqkW(coef(j).nbord(1):coef(j).nbord(2));
    % hmin before integration
    coef(j).supcoefnoint=max(coef(j).value*2^(-gamint*j));
    [coef(j).supcoef, coef(j).supcoefid]=max(coef(j).value);
    coef(j).gamma=gamint;
    coef(j).sign=sign(decime);
    nj.W(j) = length(coef(j).value);
    %% <---- ADDED 16/04/2007
    
    if doLeaders
        %-- Phase 1b: calculate the (p-) wavelet leaders at this scale
        if j==1
            decime = abs(conv_gg1(nls:2:njtemp+nls-1) ) *2^(j/2)/2^(j/Norm); %%%%%%% passage Norme L1
            
            % fractional integration
            decime = decime*2^(gamint*j);
            leaders(1).sans_voisin.value_sbord = decime;
            leaders(1).value_sbord = max([decime(1:end-2);decime(2:end-1);decime(3:end)]);
        else
            decime = abs(conv_gg1(nls:2:njtemp+nls-1) ) *2^(j/2)/2^(j/Norm) ; %%%%%%% passage Norme L1
            % fractional integration
            decime = decime*2^(gamint*j);
            length_detail = 2*length(decime);
            % leaders
            leaders(j).sans_voisin.value_sbord = max([decime  ; leaders(j-1).sans_voisin.value_sbord(1:2:length_detail); leaders(j-1).sans_voisin.value_sbord(2:2:length_detail)]);
            leaders(j).value_sbord = max([leaders(j).sans_voisin.value_sbord(1:end-2);leaders(j).sans_voisin.value_sbord(2:end-1);leaders(j).sans_voisin.value_sbord(3:end)]);
        end
        leaders(j).value_sbord = [leaders(j).value_sbord(1), leaders(j).value_sbord, leaders(j).value_sbord(end)];
        leaders(j).nbord(1)=coef(j).nbord(1)+1; leaders(j).nbord(2)=coef(j).nbord(2)-1;
        leaders(j).sans_voisin.value=leaders(j).sans_voisin.value_sbord(coef(j).nbord(1):coef(j).nbord(2));
        leaders(j).value=leaders(j).value_sbord(leaders(j).nbord(1):leaders(j).nbord(2));
        nj.L(j) = length(leaders(j).value);
        
        leaders(j).p=p;
        leaders(j).gamma=gamint;
        [leaders(j).mincoef, leaders(j).mincoefid]=min(leaders(j).value);
        [leaders(j).supcoefL, leaders(j).supcoefidL]=max(leaders(j).value);
        leaders(j).supcoefnoint=coef(j).supcoefnoint; leaders(j).supcoef=coef(j).supcoef; leaders(j).supcoefid=coef(j).supcoefid;
        coef(j).mincoef=leaders(j).mincoef; coef(j).mincoefid=leaders(j).mincoefid;
        coef(j).supcoefL=leaders(j).supcoefL; coef(j).supcoefidL=leaders(j).supcoefidL;
    end
end
end 

function [a,b]=convoluteur(c,d,e,f);
a=conv(c,e);
b=d+f;
end

function [a,b]=flippeur(c,d);
ls=max(size(c));
b=-ls+1-d;
a=fliplr(c);
end

function [a,b]=upsample1(c,d);
ls=max(size(c));
a=zeros(1,2*ls-1);
a(1:2:2*ls-1)=c;
b=2*d;
end

function [a,b]=tildeurflippeur(c,d);
ls=max(size(c));
k=d:1:ls+d-1;
b=-ls+1-d;
a=(-1).^(k).*c;
a=fliplr(a);
end

function h=rlistcoefdaub(regu);
if ~(regu<11 & regu>0),
    mess=[' Ondelette non implantee'];
    disp(mess);
    return;
else
    if regu==1,
        h=[1/sqrt(2) 1/sqrt(2)];
        %  g=[1/sqrt(2) -1/sqrt(2)];
    end
    if regu==2,
        h(1:2)=[0.482962913145 0.836516303738];
        h(3:4)=[0.224143868042 -0.129409522551];
    end
    if regu==3,
        h(1:2)=[0.332670552950 0.806891509311];
        h(3:4)=[0.459877502118 -0.135011020010];
        h(5:6)=[-0.085441273882 0.035226291882];
    end
    if regu==4,
        h(1:2)=[0.230377813309 0.714846570553];
        h(3:4)=[0.630880767930 -0.027983769417];
        h(5:6)=[-0.187034811719 0.030841381836];
        h(7:8)=[0.032883011667 -0.010597401785];
    end
    if regu==5,
        h(1:2)=[0.160102397974 0.603829269797];
        h(3:4)=[0.724308528438 0.138428145901];
        h(5:6)=[-0.242294887066 -0.032244869585];
        h(7:8)=[0.077571493840 -0.006241490213];
        h(9:10)=[-0.012580751999 0.003335725285];
    end
    if regu==6,
        h(1:2)=[0.111540743350 0.494623890398];
        h(3:4)=[0.751133908021 0.315250351709];
        h(5:6)=[-0.226264693965 -0.129766867567];
        h(7:8)=[0.097501605587 0.027522865530];
        h(9:10)=[-0.031582039318 0.000553842201];
        h(11:12)=[0.004777257511 -0.001077301085];
    end
    if regu==7,
        h(1:2)=[0.077852054085 0.396539319482];
        h(3:4)=[0.729132090846 0.469782287405];
        h(5:6)=[-0.143906003929 -0.224036184994];
        h(7:8)=[0.071309219267 0.080612609151];
        h(9:10)=[-0.038029936935 -0.016574541631];
        h(11:12)=[0.012550998556 0.000429577973];
        h(13:14)=[-0.001801640704 0.000353713800];
    end
    if regu==8,
        h(1:2)=[0.054415842243 0.312871590914];
        h(3:4)=[0.675630736297 0.585354683654];
        h(5:6)=[-0.015829105256 -0.284015542962];
        h(7:8)=[0.000472484574 0.128747426620];
        h(9:10)=[-0.017369301002 -0.044088253931];
        h(11:12)=[0.013981027917 0.008746094047];
        h(13:14)=[-0.004870352993 -0.000391740373];
        h(15:16)=[0.000675449406 -0.000117476784];
    end
    if regu==9,
        h(1:2)=[0.038077947364 0.243834674613];
        h(3:4)=[0.604823123690 0.657288078051];
        h(5:6)=[0.133197385825 -0.293273783279];
        h(7:8)=[-0.096840783223 0.148540749338];
        h(9:10)=[0.030725681479 -0.067632829061];
        h(11:12)=[0.000250947115 0.022361662124];
        h(13:14)=[-0.004723204758 -0.004281503682];
        h(15:16)=[0.001847646883 0.000230385764];
        h(17:18)=[-0.000251963189 0.000039347320];
    end
    if regu==10,
        h(1:2)=[0.026670057901 0.188176800078];
        h(3:4)=[0.52720118932 0.688459039454];
        h(5:6)=[0.281172343661 -0.249846424327];
        h(7:8)=[-0.195946274377 0.127369340336];
        h(9:10)=[0.093057364604 -0.071394147166];
        h(11:12)=[-0.029457536822 0.033212674059];
        h(13:14)=[0.003606553567 -0.010733175483];
        h(15:16)=[0.001395351747 0.001992405295];
        h(17:18)=[-0.000685856695 -0.000116466855];
        h(19:20)=[0.000093588670 -0.000013264203];
    end
end
end

function [ze,ae] = regr_cp(VV,j1,j2,nj,wtype)
jj=j1:j2;J=length(jj);njj=nj(jj);yjj=VV(jj);
wvarjj = ones(1,J);
if     wtype==1
    wvarjj = 1./njj;
end
    % use weighted regression formula in all cases
    S0 = sum(1./wvarjj) ;
    S1 = sum(jj./wvarjj) ;
    S2 = sum(jj.^2./wvarjj) ;
    wjj = (S0 * jj - S1) ./ wvarjj / (S0*S2-S1*S1) ;
    vjj = (S2 - S1 * jj) ./ wvarjj / (S0*S2-S1*S1) ;
    %  Estimate  zeta
    ze  = sum(wjj .* yjj )/log(2) ;       % zeta is just the slope, unbiased regardless of the weights
    ae     = sum(vjj .* yjj ) ;       % intercept  'a'
end

